@@ -1,5 +1,5 @@
-#!/usr/local/bin/perl
-use lib qw(. ../lib /home/tobeya/work/lib);
+#!/usr/bin/perl
+use lib qw(./lib ../lib /home/tobeya/work/lib);
 
 use Nagios::Object qw(:all);
 
@@ -16,6 +16,7 @@ elsif ( $ARGV[0] =~ /[^\d]/ ) {
 	    if ( $nf =~ /V2$/i )         { $flag = ($flag | NAGIOS_V2)         }
 	    if ( $nf =~ /V1_ONLY$/i )    { $flag = ($flag | NAGIOS_V1_ONLY)    }
 	    if ( $nf =~ /V2_ONLY$/i )    { $flag = ($flag | NAGIOS_V2_ONLY)    }
+	    if ( $nf =~ /NO_DISPLAY$/i ) { $flag = ($flag | NAGIOS_NO_DISPLAY) }
 	}
 
 	printf "Integer for flags '%s' is %d\n", join(', ',@ARGV), $flag;
@@ -29,6 +30,7 @@ else {
     push( @flags, 'V2' )         if ( ($ARGV[0] & NAGIOS_V2) == NAGIOS_V2 );
     push( @flags, 'V1_ONLY' )    if ( ($ARGV[0] & NAGIOS_V1_ONLY) == NAGIOS_V1_ONLY );
     push( @flags, 'V2_ONLY' )    if ( ($ARGV[0] & NAGIOS_V2_ONLY) == NAGIOS_V2_ONLY );
+    push( @flags, 'NO_DISPLAY' ) if ( ($ARGV[0] & NAGIOS_NO_DISPLAY) == NAGIOS_NO_DISPLAY);
 
     printf "Flags in number %d are '%s'\n", $ARGV[0], join(', ', @flags);
 }
